/*
 *  Copyright (C) 2013-2014 Katharina Breininger
 *
 *   This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import ij.plugin.PlugIn;


/**
 * This class provides simple test for the curve fitting/optimization process with the Gaussian function.
 * 
 * Please be aware that this class only checks if the class 'ScalableGaussianOptimizationFunction' works as expected by exmplarily comparing results.
 *
 */
public class TestGauss {
	// Data from Matlab
	static final double[] yvalues_1 = {2.1596387e-1, 2.3376378e-1, 2.5262625e-1, 2.7257426e-1, 2.9362725e-1, 3.1580063e-1, 3.3910545e-1, 3.6354792e-1, 3.8912908e-1, 4.1584438e-1, 4.4368334e-1, 4.7262918e-1, 5.0265855e-1, 5.3374122e-1, 5.6583986e-1, 5.9890986e-1, 6.3289916e-1, 6.6774817e-1, 7.0338972e-1, 7.3974912e-1, 7.7674422e-1, 8.1428555e-1, 8.5227659e-1, 8.9061400e-1, 9.2918802e-1, 9.6788290e-1, 1.0065774e+0, 1.0451452e+0, 1.0834559e+0, 1.1213752e+0, 1.1587662e+0, 1.1954896e+0, 1.2314050e+0, 1.2663716e+0, 1.3002491e+0, 1.3328984e+0, 1.3641832e+0, 1.3939701e+0, 1.4221301e+0, 1.4485395e+0, 1.4730806e+0, 1.4956424e+0, 1.5161221e+0, 1.5344252e+0, 1.5504665e+0, 1.5641708e+0, 1.5754734e+0, 1.5843208e+0, 1.5906708e+0, 1.5944930e+0, 1.5957691e+0, 1.5944930e+0, 1.5906708e+0, 1.5843208e+0, 1.5754734e+0, 1.5641708e+0, 1.5504665e+0, 1.5344252e+0, 1.5161221e+0, 1.4956424e+0, 1.4730806e+0, 1.4485395e+0, 1.4221301e+0, 1.3939701e+0, 1.3641832e+0, 1.3328984e+0, 1.3002491e+0, 1.2663716e+0, 1.2314050e+0, 1.1954896e+0, 1.1587662e+0, 1.1213752e+0, 1.0834559e+0, 1.0451452e+0, 1.0065774e+0, 9.6788290e-1, 9.2918802e-1, 8.9061400e-1, 8.5227659e-1, 8.1428555e-1, 7.7674422e-1, 7.3974912e-1, 7.0338972e-1, 6.6774817e-1, 6.3289916e-1, 5.9890986e-1, 5.6583986e-1, 5.3374122e-1, 5.0265855e-1, 4.7262918e-1, 4.4368334e-1, 4.1584438e-1, 3.8912908e-1, 3.6354792e-1, 3.3910545e-1, 3.1580063e-1, 2.9362725e-1, 2.7257426e-1, 2.5262625e-1, 2.3376378e-1};
	static final double[] xvalues_1 = getXValues(yvalues_1.length, 0, 0.01);
	static final double mean_1 = 0.5;
	static final double stddev_1 = 0.25;
	static ScalableGaussianOptimizationFunction gauss = new ScalableGaussianOptimizationFunction();
	
	public static void main(String[] args) {
		boolean works;
		
		works = testIfCorrectTestValues();
		if(!works)
			System.err.println("Test failed - at least one test value was not calculated correctly - test setup is not correct!");
		else
			System.out.println("Test 'Test Values Correct' successful");
		
		//Basic testing of calculation methods:
		works = testIfGaussianIsCalculatedCorrectly();
		if(!works)
			System.err.println("Test failed - at least one value of the Gauss function was not calculated correctly");
		else
			System.out.println("Test 'Gauss' successful");

		
		
	}
	
	
	private static double[] getXValues(int length, double start, double stepsize) {
		double[] xvalues = new double[length];
		for(int i = 0; i < length; ++i)
			xvalues[i] = start+i*stepsize;
		return xvalues;
	}
	
	
	public void run(String arg) {

		
		int valueSize = 100;
		double[] xValues = new double[valueSize];
		for(int i = 0; i < valueSize; ++i){
			xValues[i] = i;
		}
		double[] yValues = getValuesOfTripleGaussian(xValues);
	}

	private double[] getValuesOfTripleGaussian(double[] xValues) {
		double[] result = new double[xValues.length];
		
		return result;
	}
	
	
	static double evaluateTripleGaussian(double x, double[] params) {
		double result = 0;
		for(int i = 0; i < 3; ++i) {
			result += evaluateGaussian(x, params[i*2], params[i*2+1]);
		}
		return result;
	}
	
	static double evaluateGaussian (double x, double mean, double standardDev, double scaling) {
		return gauss.evaluateGaussian(x, mean, standardDev, scaling);
	}

	
	static double evaluateGaussian (double x, double mean, double standardDev) {
		return evaluateGaussian(x, mean, standardDev, 1);
		
	}

	static double testTestValues (double x, double mean, double standardDev) {
	
		return 1/(standardDev*Math.sqrt(2*Math.PI))*Math.pow(Math.E, -0.5*((x-mean)*(x-mean)/(standardDev*standardDev)));
	
	}
	
	// Test functions
	
	static boolean testIfCorrectTestValues() {
		for(int i = 0; i <yvalues_1.length; ++i) {
			if(testTestValues(xvalues_1[i], mean_1, stddev_1) + 0.00001 < yvalues_1[i] ||testTestValues(xvalues_1[i], mean_1, stddev_1) - 0.00001 > yvalues_1[i]) {
				System.err.println("Wrong Value calculated!");
				return false;
			}
		}
		return true;
	}
	
	static boolean testIfGaussianIsCalculatedCorrectly() {
		
		for(int i = 0; i <yvalues_1.length; ++i) {
			if(evaluateGaussian(xvalues_1[i], mean_1, stddev_1) + 0.00001 < yvalues_1[i] ||evaluateGaussian(xvalues_1[i], mean_1, stddev_1) - 0.00001 > yvalues_1[i]) {
				System.err.println("Wrong Value calculated!");
				return false;
			}
		}
		return true;
	}
}
